// Do the integral function.

#include "stdafx.h"
#include "defs.h"

static void integral_f(void);
static void integral_of_sum(void);
static void integral_of_product(void);
static void integral_of_power(void);
static void integral_of_derivative1(void);
static void integral_of_derivative2(void);
static void integral_of_integral(void);
static void integral_of_form(void);
static void scan_integrals(void);
extern int match(U *, U *, U *, int, int);
extern int isnegativeterm(U *);

#define F p3
#define X p4
#define N p5

void
eval_integral(void)
{
	int i, n;

	// evaluate 1st arg to get function F

	p1 = cdr(p1);
	push(car(p1));
	eval();

	// evaluate 2nd arg and then...

	// example		result of 2nd arg	what to do
	//
	// integral(f)		nil			guess X, N = nil
	// integral(f,2)	2			guess X, N = 2
	// integral(f,x)	x			X = x, N = nil
	// integral(f,x,2)	x			X = x, N = 2
	// integral(f,x,y)	x			X = x, N = y

	p1 = cdr(p1);
	push(car(p1));
	eval();

	p2 = pop();
	if (p2 == nil) {
		guess();
		push(nil);
	} else if (isnum(p2)) {
		guess();
		push(p2);
	} else {
		push(p2);
		p1 = cdr(p1);
		push(car(p1));
		eval();
	}


	N = pop();
	X = pop();
	F = pop();

	if (!isinteger(N) && N != nil) {
		push_symbol(INTEGRAL);
		push(F);
		eval();
		push(X);
		push(N);
		list(4);
		return;
	}

	while (1) {

		// N might be a symbol instead of a number

		if (isnum(N)) {
			push(N);
			n = pop_integer();
			if (n == (int) 0x80000000)
				stop("nth integral: check n");
		} else
			n = 1;

		push(F);

		if (n >= 0) {
			for (i = 0; i < n; i++) {
				push(X);
				integral();
			}
		} else {
			n = -n;
			for (i = 0; i < n; i++) {
				push(X);
				derivative();
			}
		}

		F = pop();

		// if N is nil then arglist is exhausted

		if (N == nil)
			break;

		// otherwise...

		// N		arg1		what to do
		//
		// number	nil		break
		// number	number		N = arg1, continue
		// number	symbol		X = arg1, N = arg2, continue
		//
		// symbol	nil		X = N, N = nil, continue
		// symbol	number		X = N, N = arg1, continue
		// symbol	symbol		X = N, N = arg1, continue

		if (isnum(N)) {
			p1 = cdr(p1);
			push(car(p1));
			eval();
			N = pop();
			if (N == nil)
				break;		// arglist exhausted
			if (isnum(N))
				;		// N = arg1
			else {
				X = N;		// X = arg1
				p1 = cdr(p1);
				push(car(p1));
				eval();
				N = pop();	// N = arg2
			}
		} else {
			X = N;			// X = N
			p1 = cdr(p1);
			push(car(p1));
			eval();
			N = pop();		// N = arg1
		}
	}

	push(F); // final result
}


void
integral(void)
{
	save();
	integral_f();
	restore();
}

static void
integral_f(void)
{
	if (table_of_integrals == nil)
		scan_integrals();


	p2 = pop();
	p1 = pop();

	if (istensor(p1)) {
		integral_tensor_scalar();
		return;
	}
	if (!istensor(p2) && isnegativeterm(p2)){
		push_symbol(ANTIDER);
		push(p1);
		push(p2);
		negate();
		list(3);
		eval();
/*	 	integral(); */
		push_integer(-1);
		multiply();
	}
	else {
		if (car(p1) == symbol(ADD))
			integral_of_sum();
		else if (car(p1) == symbol(MULTIPLY))
			integral_of_product();
		else if (car(p1) == symbol(POWER) && ibpmode ==1 && isinteger(caddr(p1)))
			integral_of_power();  
		else if (car(p1) == symbol(INTEGRAL) && caddr(p1) != p2)
			integral_of_integral();
		else if (car(p1) == symbol(DERIVATIVE)){
			if (caddr(p1) == p2)
				integral_of_derivative1();
			else
				integral_of_derivative2();}
		else 
			integral_of_form();
	}
}


static void
integral_of_sum(void)
{
	p1 = cdr(p1);
	push_symbol(ANTIDER);
	push(car(p1));
	push(p2);
	list(3);
	eval();
/* 	integral(); */
	p1 = cdr(p1);
	while (iscons(p1)) {
		push_symbol(ANTIDER);
		push(car(p1));
		push(p2);
		list(3);
		eval();
/*		integral(); */
		add();
		p1 = cdr(p1);
	}
}

static void
integral_of_product(void)
{
	int h1, h2;


	h1 = tos;
	p3 = cdr(p1);
	while (iscons(p3)) {
		if (!find(car(p3), p2))
			push(car(p3));
		p3 = cdr(p3);
	}

	h2 = tos;

	p3 = cdr(p1);
	while (iscons(p3)) {
		if (find(car(p3), p2)) {
			push(car(p3));}
		if (caar(p3) == symbol(POWER) && cadar(p3) == p2 && !find(caddar(p3),p2)){
			push(caddar(p3));
			p6 = pop();
		}
		if (car(p3) == p2){
			push_integer(1);
			p6 = pop();
		}
		p3 = cdr(p3);
	}

	
	if (tos - h2 == 0)
		push(p2);
	else {  
		multiply_all(tos - h2);
		p1 = pop();
		if (!iszero(p6) && p6!=nil && ibpmode == 1){
			push(p1);
			push(p1);
			p3=pop();
			push(p2);
			push(p6);
			power();
			divide();
			p1=pop();
		}
		if (!isplusone(p1)) {
			if (!isnegativeterm(p6) && p6!=nil && ibpmode == 1) {
				push(p2);
				push(p6);
				power();
				push_symbol(ANTIDER);
				push(p1);
				push(p2);
				list(3);
//				integral();
				multiply();
				push_symbol(ANTIDER);
				push(p2);
				push(p6);
				power();
				push(p2);
				derivative();
				push_symbol(ANTIDER);
				push(p1);
				push(p2);
				list(3);
//				integral();
				multiply();
				push(p2);
				list(3);
				negate();
				add();}
			else
			if (isnegativeterm(p6) && p6!=nil && ibpmode == 1) {
				push(p2);
				push(p6);
				power();
				push(p2);
				integral();
				push(p1);
				multiply();
				push_symbol(ANTIDER);
				push(p2);
				push(p6);
				power();
				push(p2);
				integral();
				push(p1);
				push(p2);
				derivative();
				multiply();
				push(p2);
				list(3);
				negate();
				add();}
			else 	{
				push(p1);
				numerator();
				p3=pop();
				push(p1);
				denominator();
				p4=pop();
				push(p4);	
				push(p2);
				derivative();
				p5=pop();
				if (!iszero(p5) && ibpmode == 1) {
					push(p3);
					push(p5);
					divide();
					p5=pop();
					if (!find(p5,p2)) {
						push(p4);
						logarithm();
						push(p5);
						multiply();}
					else {
					integral_of_form();}}
				else integral_of_form();}
		}
		else {  
			push(p3);
			push(p2);
			integral();}
	}
	multiply_all(tos - h1);
}

static void
integral_of_power(void) {

	if (car(p1)==symbol(POWER) && 
		cadr(p1) != p2 && 
		!find(caddr(p1),p2) &&
		!isminusone(caddr(p1))){
		if (!isnegativeterm(caddr(p1))) {
			push_symbol(ANTIDER);
			push(cadr(p1));
			push(p2);
			list(3);
//			integral();
			push(cadr(p1));
			push(caddr(p1));
			push_integer(-1);
			add();
			power();
			multiply();
			push_symbol(ANTIDER);
			push_symbol(ANTIDER);
			push(cadr(p1));
			push(p2);
			list(3);
//			integral();
			push(cadr(p1));
			push(caddr(p1));
			push_integer(-1);
			add();
			power();
			push(p2);
			derivative();
			multiply();
			push(p2);
			list(3);
			negate();
			add();}
		else	{
			push_symbol(ANTIDER);
			push(cadr(p1));
			inverse();
			push(p2);
			list(3);
//			integral();
			push(cadr(p1));
			push(caddr(p1));
			push_integer(1);
			add();
			power();
			multiply();
			push_symbol(ANTIDER);
			push_symbol(ANTIDER);
			push(cadr(p1));
			inverse();
			push(p2);
			list(3);
//			integral();
			push(cadr(p1));
			push(caddr(p1));
			push_integer(1);
			add();
			power();
			push(p2);
			derivative();
			multiply();
			push(p2);
			list(3);
			negate();
			add();}}
	else
		integral_of_form();
}



static void
integral_of_derivative1(void)
{
	push(cadr(p1));
}

static void
integral_of_derivative2(void)
{
	push_symbol(ANTIDER);
	push(cadr(p1));
	push(p2);
	list(3);
/*	integral(); */
	push(caddr(p1));
	derivative();
}


static void
integral_of_integral(void)
{
	push(cadr(p1));
	push(p2);
	integral(); 
	push(caddr(p1));
	list(3);
	integral(); 
}

static void
integral_of_form(void)
{
	int h;

	// save meta symbols

	push(meta_a->u.sym.binding);
	push(meta_b->u.sym.binding);
	push(meta_c->u.sym.binding);
	push(meta_n->u.sym.binding);
	push(meta_x->u.sym.binding);

	meta_x->u.sym.binding = p2;

	h = tos;

	push(one);

	push(p1);
	push(p2);

	distill();

	p3 = table_of_integrals;

	while (iscons(p3)) {

		p4 = car(p3);

		if (match(p1, car(p4), cddr(p4), h, tos))
			break;

		p3 = cdr(p3);
	}

	if (iscons(p3)) {

		push(cadr(p4));
		eval();
	} else {

		push_symbol(INTEGRAL);
		push(p1);
		push(p2);
		list(3);
	}

	p3 = pop();

	tos = h;

	// restore meta symbols

	meta_x->u.sym.binding = pop();
	meta_n->u.sym.binding = pop();
	meta_c->u.sym.binding = pop();
	meta_b->u.sym.binding = pop();
	meta_a->u.sym.binding = pop();

	push(p3);
}

char *integrals[] = {
	
	"heaviside(x)",
	"x*heaviside(x)",
	NULL,

	"dirac(a*x)",
	"sgn(a*x)/(2*a)",
	NULL,

	"dirac(a*x+b)",
	"sgn(a*x+b)/(2*a)",
	NULL,

	"sgn(a*x)",
	"abs(a*x)/a",
	NULL,


	"sgn(a*x+b)",
	"abs(a*x+b)/b",
	NULL,

	"sgn(x)/abs(x)",
	"log(x)",
	NULL,
	
	"abs(x)",
	"x*abs(x)/2",
	NULL,

	"a",
	"a*x",
	NULL,

	"1/x",
//	"log(abs(x))",
	"log(x)",
	NULL,

	"1/(a+b*x)",
	"1/b*log(abs(a+b*x))",
	"opemode==1",
	NULL,

	"1/c*(a+b*x)",
	"1/(c*b)*log(abs(c*(a+b*x)))",
	"opemode==1",
	NULL,

	"1/(b*x)",
	"1/b*log(abs(b*x))",
	"opemode==1",
	NULL,


	"x^a",
	"x^(a+1)/(a+1)",
	"not(a==-1)",
	NULL,

	"(a*x+b)^n",
	"(a*x+b)^(n+1)/(a*(n+1))",
	"not(n==-1)",
	NULL,

	"1/(a*x^2+b*x+c)",
	"2*arctan((2*a*x+b)/sqrt(-b^2+4*a*c))/sqrt(-b^2+4*a*c)",
	"-4*a*c+b^2<0",
	NULL,
	
	"1/(a*x^2+b*x+c)",
	"2*argth((2*a*x+b)/sqrt(b^2-4*a*c))/sqrt(b^2-4*a*c)",
	"-4*a*c+b^2>0",
	NULL,
	
	"1/(a*x^2+b*x+c)",
	"-2/(2*a*x+b)",
	"4*a*c==b^2",
	NULL,

	"1/(a*x^2+b*x+c)",
	"log((2*a*x+b-sqrt(b^2-4*a*c))/(2*a*x+b+sqrt(b^2-4*a*c)))/sqrt(b^2-4*a*c)",
	NULL,

	"x^n/(a*x^2+b*x+c)",
	"x^n*antider(1/(a*x^2+b*x+c),x)-n*antider(antider(1/(a*x^2+b*x+c),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a*x^2+b*x+c)^(1/2)",
	"log((2*a*x+b)/sqrt(a)+2*sqrt(a*x^2+b*x+c))/sqrt(a)",
	"not(4*a*c==b^2)",
	NULL,
	
	"x/(a*x^2+b*x+c)^(1/2)",
	"sqrt(a*x^2+b*x+c)/a-b*log((2*a*x+b)/sqrt(a)+2*sqrt(a*x^2+b*x+c))/(2*a^(3/2)))",
	NULL,

	"x^2/(a*x^2+b*x+c)^(1/2)",
	"(2*sqrt(a)*(-3*b+2*a*x)*sqrt(a*x^2+b*x+c)+(3*b^2-4*a*c)*log((2*a*x+b)/sqrt(a)+2*sqrt(a*x^2+b*x+c)))/(8*a^(5/2)))",
	"not(4*a*c==b^2)",
	NULL,

	"1/(a*x^2+b*x+c)^(1/2)",
	"antider(1/(x+b/(2*a)),x)/sqrt(a)",
	"4*a*c==b^2",
	NULL,
	
	"x/(a*x^2+b*x+c)^(1/2)",
	"antider(x/(x+b/(2*a)),x)/sqrt(a)",
	"4*a*c==b^2",
	NULL,

	"x^2/(a*x^2+b*x+c)^(1/2)",
	"antider(x^2/(x+b/(2*a)),x)/sqrt(a)",
	"4*a*c==b^2",
	NULL,

	"x^n/(a*x^2+b*x+c)^(1/2)",
	"x^n*antider(1/(a*x^2+b*x+c)^(1/2),x)-n*antider(antider(1/(a*x^2+b*x+c)^(1/2),x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"(a*x^2+b*x+c)^(1/2)",
	"(2*a*x+b)*sqrt(a*x^2+b*x+c)/(4*a)+antider((a*x^2+b*x+c)^(-1/2),x)*((4a*c-b^2)/8*a)",
	"not(4*a*c==b^2)",
	NULL,
	
	"(a*x^2+b*x+c)^(1/2)",
	"antider((x+b/(2*a)),x)*sqrt(a)",
	"4*a*c==b^2",
	NULL,
	
	"x^n*(a*x^2+b*x+c)^(1/2)",
	"x^n*antider((a*x^2+b*x+c)^(1/2),x)-n*antider(antider((a*x^2+b*x+c)^(1/2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a*x^2+c)",
	"arctan((a*x)/sqrt(a*c))/sqrt(a*c)",
	"a*c>0",
	NULL,
	
	"1/(a*x^2+c)",
	"-argth((a*x)/sqrt(-a*c))/sqrt(-a*c)",
	"a*c<0",
	NULL,
	
	"x/(a*x^2+c)",
	"log(c+a*x^2)/(2*a)",
	NULL,

	"x^n/(a*x^2+c)",
	"x^n*antider(1/(a*x^2+c),x)-n*antider(antider(1/(a*x^2+c),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"1/(a*x^2+c)^(1/2)",
	"log((2*a*x)/sqrt(a)+2*sqrt(a*x^2+c))/sqrt(a)",
	NULL,
	
	"x/(a*x^2+c)^(1/2)",
	"sqrt(a*x^2+c))/a-b*log((2*a*x)/sqrt(a)+2*sqrt(a*x^2+c))/(2*a^(3/2)))",
	NULL,
	
	"x^2/(a*x^2+c)^(1/2)",
	"(2*sqrt(a)*(2*a*x)*sqrt(a*x^2+c)+(-4*a*c)*log((2*a*x)/sqrt(a)+2*sqrt(a*x^2+c)))/(8*a^(5/2)))",
	NULL,

	"x^n/(a*x^2+c)^(1/2)",
	"x^n*antider(1/(a*x^2+c)^(1/2),x)-n*antider(antider(1/(a*x^2+c)^(1/2),x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"(a*x^2+c)^(1/2)",
	"(2*a*x)*sqrt(a*x^2+c)/(4*a)+antider((a*x^2+c)^(-1/2),x)*(c/2)",
	NULL,
	
	"x^n*(a*x^2+c)^(1/2)",
	"x^n*antider((a*x^2+c)^(1/2),x)-n*antider(antider((a*x^2+c)^(1/2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x)",
	"1/a*exp(a*x)",
	NULL,

	"x^n*exp(a*x)",
	"x^n*antider(exp(a*x),x)-n*antider(antider(exp(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b*x)",
	"1/(a+b)*exp(a*x)*exp(b*x)",
	NULL,

	"x^n*a^(b*x)",
	"x^n*antider(a^(b*x),x)-n*antider(antider(a^(b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"x^n*exp(a*x+b*x)",
	"x^n*antider(exp(a*x+b*x),x)-n*antider(antider(exp(a*x+b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"a^(b*x)",
	"1/(b*log(a))*a^(b*x)",
	NULL,
	
	"x^n*a^(b*x)",
	"x^n*antider(a^(b*x),x)-n*antider(antider(a^(b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"(a^(b*x))^c",
	"1/(b*c*log(a))*(a^(b*x))^c",
	NULL,
	
	"x^n*(a^(b*x))^c",
	"x^n*antider((a^(b*x))^c,x)-n*antider(antider(a^(b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"a^(b*x+c)",
	"1/(b*log(a))*a^(b*x+c)",
	NULL,

	"x^n*a^(b*x+c)",
	"x^n*antider(a^(b*x+c),x)-n*antider(antider(a^(b*x+c),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"(a^(b*x+c))^n",
	"1/(b*n*log(a))*(a^(b*x+c))^n",
	NULL,

	"exp(a*x+b)",
	"1/a*exp(a*x+b)",
	NULL,
		
	"x^n*exp(a*x+b)",
	"x^n*antider(exp(a*x+b),x)-n*antider(antider(exp(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a*x)^b*sin(a*x)",
	"-cos(a*x)^(b+1)/(a*(b+1))",
	"(b>0)*(isinteger(b))",
	NULL,
	
	"x^n*cos(a*x)^b*sin(a*x)",
	"x^n*antider(cos(a*x)^b*sin(a*x),x)-n*antider(antider(cos(a*x)^b*sin(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))*(b>0)*(isinteger(b))",
	NULL,

	"sin(a*x)^b*cos(a*x)",
	"sin(a*x)^(b+1)/(a*(b+1))",
	"(b>0)*(isinteger(b))",
	NULL,
	
	"x^n*sin(a*x)^b*cos(a*x)",
	"x^n*antider(sin(a*x)^b*cos(a*x),x)-n*antider(antider(sin(a*x)^b*cos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))*(b>0)*(isinteger(b))",
	NULL,

	"sin(a*x)^b*cos(a*x)^c",
	"-sin(a*x)^(b-1)*cos(a*x)^(c-1)/(a*(b+c))+(b-1)*antider(sin(a*x)^(b-2)*cos(a*x)^c,x)/(b+c)",
	"(b>0)*(isinteger(b))*(c>0)*(isinteger(c))",
	NULL,
	
	"x^n*sin(a*x)^b*cos(a*x)^c",
	"x^n*antider(sin(a*x)^b*cos(a*x)^c,x)-n*antider(antider(sin(a*x)^b*cos(a*x)^c,x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))*(b>0)*(isinteger(b))*(c>0)*(isinteger(c))",
	NULL,

	"a^x",
	"a^x/log(a)",
/*	"a>0",*/
	NULL,

	"x^n*a^x",
	"x^n*antider(a^x,x)-n*antider(antider(a^x,x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+x^2)",
	"1/sqrt(a)*arctan(x/sqrt(a))",
/*	"a>0",*/
	NULL,

	"x^n/(a+x^2)",
	"x^n*antider(1/(a+x^2),x)-n*antider(antider(1/(a+x^2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a-x^2)",
	"1/sqrt(a)*argth(x/sqrt(a))",
	NULL,

	"x^n/(a-x^2)",
	"x^n*antider(1/(a-x^2),x)-n*antider(antider(1/(a-x^2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/sqrt(a-x^2)",
	"arcsin(x/(sqrt(a)))",
	NULL,

	"x^n/sqrt(a-x^2)",
	"x^n*antider(1/sqrt(a-x^2),x)-n*antider(antider(1/sqrt(a-x^2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/sqrt(a+x^2)",
	"log(x+sqrt(a+x^2))",
	NULL,

	"x^n/sqrt(a+x^2)",
	"x^n*antider(1/sqrt(a+x^2),x)-n*antider(antider(1/sqrt(a+x^2),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+b*x)",
	"1/b*log(a+b*x)",
	NULL,

	"x/(a+b*x)",
	"x/b-a*log(a+b*x)/b/b",
	NULL,

	"x^2/(a+b*x)",
	"1/b^2*(1/2*(a+b*x)^2-2*a*(a+b*x)+a^2*log(a+b*x))",
	NULL,

	"x^n/(a+b*x)",
	"x^n*antider(1/(a+b*x),x)-n*antider(antider(1/(a+b*x),x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"1/(a+b*x)^2",
	"-1/(b*(a+b*x))",
	NULL,

	"x/(a+b*x)^2",
	"1/b^2*(log(a+b*x)+a/(a+b*x))",
	NULL,

	"x^2/(a+b*x)^2",
	"1/b^3*(a+b*x-2*a*log(a+b*x)-a^2/(a+b*x))",
	NULL,

	"x^n/(a+b*x)^2",
	"x^n*antider(1/(a+b*x)^2,x)-n*antider(antider(1/(a+b*x)^2,x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"1/(a+b*x)^3",
	"-1/(2*b)*1/(a+b*x)^2",
	NULL,

	"x/(a+b*x)^3",
	"-(a+2*b*x)/(2*b^2)*1/(a+b*x)^2",
	NULL,

	"x^2/(a+b*x)^3",
	"1/b^3*(log(a+b*x)+2*a/(a+b*x)-1/2*a^2/(a+b*x)^2)",
	NULL,

	"x^n/(a+b*x)^3",
	"x^n*antider(1/(a+b*x)^3,x)-n*antider(antider(1/(a+b*x)^3,x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/x*1/(a+b*x)",
	"-1/a*log((a+b*x)/x)",
	NULL,
// 38
	"1/x*1/(a+b*x)^2",
	"1/a*1/(a+b*x)-1/a^2*log((a+b*x)/x)",
	NULL,
// 39
	"1/x*1/(a+b*x)^3",
	"1/a^3*(1/2*((2*a+b*x)/(a+b*x))^2+log(x/(a+b*x)))",
	NULL,
// 40
	"1/x^2*1/(a+b*x)",
	"-1/(a*x)+b/a^2*log((a+b*x)/x)",
	NULL,
// 41
	"1/x^3*1/(a+b*x)",
	"(2*b*x-a)/(2*a^2*x^2)+b^2/a^3*log(x/(a+b*x))",
	NULL,
// 42
	"1/x^2*1/(a+b*x)^2",
	"-(a+2*b*x)/(a^2*x*(a+b*x))+2*b/a^3*log((a+b*x)/x)",
	NULL,
// 60
	"1/(a+b*x^2)",
	"1/sqrt(a*b)*arctan(x*sqrt(a*b)/a)",
	"a*b>0",
	NULL,
// 61
	"1/(a+b*x^2)",
	"1/(2*sqrt(-a*b))*log((a+x*sqrt(-a*b))/(a-x*sqrt(-a*b)))",
	"a*b<0",
	NULL,
// 62 is the same as 60
// 63
	"x/(a+b*x^2)",
	"1/2*1/b*log(a+b*x^2)",
	NULL,
//64
	"x^2/(a+b*x^2)",
	"x/b-a/b*antider(1/(a+b*x^2),x)",
	NULL,
//65
	"1/(a+b*x^2)^2",
	"x/(2*a*(a+b*x^2))+1/2*1/a*antider(1/(a+b*x^2),x)",
	NULL,
//66 is covered by 61
//70
	"1/x*1/(a+b*x^2)",
	"1/2*1/a*log(x^2/(a+b*x^2))",
	NULL,
//71
	"1/x^2*1/(a+b*x^2)",
	"-1/(a*x)-b/a*antider(1/(a+b*x^2),x)",
	NULL,
//74
	"1/(a+b*x^3)",
	"1/3*1/a*(a/b)^(1/3)*(1/2*log(((a/b)^(1/3)+x)^3/(a+b*x^3))+sqrt(3)*arctan((2*x-(a/b)^(1/3))*(a/b)^(-1/3)/sqrt(3)))",
	NULL,
//76
	"x^2/(a+b*x^3)",
	"1/3*1/b*log(a+b*x^3)",
	NULL,
//77
	"1/(a+b*x^4)",
	"1/2*1/a*(a/b/4)^(1/4)*(1/2*log((x^2+2*(a/b/4)^(1/4)*x+2*(a/b/4)^(1/2))/(x^2-2*(a/b/4)^(1/4)*x+2*(a/b/4)^(1/2)))+arctan(2*(a/b/4)^(1/4)*x/(2*(a/b/4)^(1/2)-x^2)))",
	"a*b>0",
	NULL,
//78
	"1/(a+b*x^4)",
	"1/2*(-a/b)^(1/4)/a*(1/2*log((x+(-a/b)^(1/4))/(x-(-a/b)^(1/4)))+arctan(x*(-a/b)^(-1/4)))",
	"a*b<0",
	NULL,
//79
	"x/(a+b*x^4)",
	"1/2*sqrt(b/a)/b*arctan(x^2*sqrt(b/a))",
	"a*b>0",
	NULL,
//80
	"x/(a+b*x^4)",
	"1/4*sqrt(-b/a)/b*log((x^2-sqrt(-a/b))/(x^2+sqrt(-a/b)))",
	"a*b<0",
	NULL,
//81
	"x^2/(a+b*x^4)",
	"1/4*1/b*(a/b/4)^(-1/4)*(1/2*log((x^2-2*(a/b/4)^(1/4)*x+2*sqrt(a/b/4))/(x^2+2*(a/b/4)^(1/4)*x+2*sqrt(a/b/4)))+arctan(2*(a/b/4)^(1/4)*x/(2*sqrt(a/b/4)-x^2)))",
	"a*b>0",
	NULL,
//82
	"x^2/(a+b*x^4)",
	"1/4*1/b*(-a/b)^(-1/4)*(log((x-(-a/b)^(1/4))/(x+(-a/b)^(1/4)))+2*arctan(x*(-a/b)^(-1/4)))",
	"a*b<0",
	NULL,
//83
	"x^3/(a+b*x^4)",
	"1/4*1/b*log(a+b*x^4)",
	NULL,

	"sqrt(a+b*x)",
	"2/3*1/b*sqrt((a+b*x)^3)",
	NULL,
//125
	"x*sqrt(a+b*x)",
	"-2*(2*a-3*b*x)*sqrt((a+b*x)^3)/15/b^2",
	NULL,
//126
	"x^2*sqrt(a+b*x)",
	"2*(8*a^2-12*a*b*x+15*b^2*x^2)*sqrt((a+b*x)^3)/105/b^3",
	NULL,

	"sqrt(a+b*x)/x",
	"2*sqrt(a+b*x)+a*antider(1/x*1/sqrt(a+b*x),x)",
	NULL,
//129
	"sqrt(a+b*x)/x^2",
	"-sqrt(a+b*x)/x+b/2*antider(1/x*1/sqrt(a+b*x),x)",
	NULL,
//131
	"1/sqrt(a+b*x)",
	"2*sqrt(a+b*x)/b",
	NULL,
//132
	"x/sqrt(a+b*x)",
	"-2/3*(2*a-b*x)*sqrt(a+b*x)/b^2",
	NULL,
//133
	"x^2/sqrt(a+b*x)",
	"2/15*(8*a^2-4*a*b*x+3*b^2*x^2)*sqrt(a+b*x)/b^3",
	NULL,

	"1/x*1/sqrt(a+b*x)",
	"1/sqrt(a)*log((sqrt(a+b*x)-sqrt(a))/(sqrt(a+b*x)+sqrt(a)))",
	"a>0",
	NULL,
//136
	"1/x*1/sqrt(a+b*x)",
	"2/sqrt(-a)*arctan(sqrt(-(a+b*x)/a))",
	"a<0",
	NULL,
//137
	"1/x^2*1/sqrt(a+b*x)",
	"-sqrt(a+b*x)/a/x-1/2*b/a*antider(1/x*1/sqrt(a+b*x),x)",
	NULL,
//156
	"sqrt(x^2+a)",
	"1/2*(x*sqrt(x^2+a)+a*log(x+sqrt(x^2+a)))",
	NULL,

	"1/sqrt(x^2+a)",
	"log(x+sqrt(x^2+a))",
	NULL,
//158
	"1/x*1/sqrt(x^2+a)",
	"arcsec(x/sqrt(-a))/sqrt(-a)",
	"a<0",
	NULL,
//159
	"1/x*1/sqrt(x^2+a)",
	"-1/sqrt(a)*log((sqrt(a)+sqrt(x^2+a))/x)",
	"a>0",
	NULL,
//160
	"sqrt(x^2+a)/x",
	"sqrt(x^2+a)-sqrt(a)*log((sqrt(a)+sqrt(x^2+a))/x)",
	"a>0",
	NULL,
//161
	"sqrt(x^2+a)/x",
	"sqrt(x^2+a)-sqrt(-a)*arcsec(x/sqrt(-a))",
	"a<0",
	NULL,
//162
	"x/sqrt(x^2+a)",
	"sqrt(x^2+a)",
	NULL,
//163
	"x*sqrt(x^2+a)",
	"1/3*sqrt((x^2+a)^3)",
	NULL,

	"sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"1/4*(x*sqrt((x^2+a^(1/3))^3)+3/2*a^(1/3)*x*sqrt(x^2+a^(1/3))+3/2*a^(2/3)*log(x+sqrt(x^2+a^(1/3))))",
	NULL,

	// match doesn't work for the following
	"sqrt(-a+x^6-3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"1/4*(x*sqrt((x^2-a^(1/3))^3)-3/2*a^(1/3)*x*sqrt(x^2-a^(1/3))+3/2*a^(2/3)*log(x+sqrt(x^2-a^(1/3))))",
	NULL,
//165
	"1/sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"x/a^(1/3)/sqrt(x^2+a^(1/3))",
	NULL,
//166
	"x/sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"-1/sqrt(x^2+a^(1/3))",
	NULL,
//167
	"x*sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"1/5*sqrt((x^2+a^(1/3))^5)",
	NULL,
//168
	"x^2*sqrt(x^2+a)",
	"1/4*x*sqrt((x^2+a)^3)-1/8*a*x*sqrt(x^2+a)-1/8*a^2*log(x+sqrt(x^2+a))",
	NULL,
//169
	"x^3*sqrt(x^2+a)",
	"(1/5*x^2-2/15*a)*sqrt((x^2+a)^3)",
	NULL,

	"x^2/sqrt(x^2+a)",
	"1/2*x*sqrt(x^2+a)-1/2*a*log(x+sqrt(x^2+a))",
	NULL,
//172
	"x^3/sqrt(x^2+a)",
	"1/3*sqrt((x^2+a)^3)-a*sqrt(x^2+a)",
	NULL,

	"1/x^2*1/sqrt(x^2+a)",
	"-sqrt(x^2+a)/a/x",
	NULL,
//174
	"1/x^3*1/sqrt(x^2+a)",
	"-1/2*sqrt(x^2+a)/a/x^2+1/2*log((sqrt(a)+sqrt(x^2+a))/x)/a^(3/2)",
	"a>0",
	NULL,
//175
	"1/x^3*1/sqrt(x^2-a)",
	"1/2*sqrt(x^2-a)/a/x^2+1/2*1/(a^(3/2))*arcsec(x/(a^(1/2)))",
	"a>0",
	NULL,
//176+
	"x^2*sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"1/6*x*sqrt((x^2+a^(1/3))^5)"
	"-1/24*a^(1/3)*x*sqrt((x^2+a^(1/3))^3)"
	"-1/16*a^(2/3)*x*sqrt(x^2+a^(1/3))"
	"-1/16*a*log(x+sqrt(x^2+a^(1/3)))",
	"a>0",
	NULL,
//176-
	"x^2*sqrt(-a-3*a^(1/3)*x^4+3*a^(2/3)*x^2+x^6)",
	"1/6*x*sqrt((x^2-a^(1/3))^5)"
	"+1/24*a^(1/3)*x*sqrt((x^2-a^(1/3))^3)"
	"-1/16*a^(2/3)*x*sqrt(x^2-a^(1/3))"
	"+1/16*a*log(x+sqrt(x^2-a^(1/3)))",
	"a>0",
	NULL,
//177+
	"x^3*sqrt(a+x^6+3*a^(1/3)*x^4+3*a^(2/3)*x^2)",
	"1/7*sqrt((x^2+a^(1/3))^7)-1/5*a^(1/3)*sqrt((x^2+a^(1/3))^5)",
	"a>0",
	NULL,
//177-
	"x^3*sqrt(-a-3*a^(1/3)*x^4+3*a^(2/3)*x^2+x^6)",
	"1/7*sqrt((x^2-a^(1/3))^7)+1/5*a^(1/3)*sqrt((x^2-a^(1/3))^5)",
	"a>0",
	NULL,
//200+
	"sqrt(a-x^2)",
	"1/2*(x*sqrt(a-x^2)+a*arcsin(x/sqrt(abs(a))))",
	NULL,

	"1/x*1/sqrt(a-x^2)",
	"-1/sqrt(a)*log((sqrt(a)+sqrt(a-x^2))/x)",
	"a>0",
	NULL,
//203
	"sqrt(a-x^2)/x",
	"sqrt(a-x^2)-sqrt(a)*log((sqrt(a)+sqrt(a-x^2))/x)",
	"a>0",
	NULL,
//204
	"x/sqrt(a-x^2)",
	"-sqrt(a-x^2)",
	NULL,

	"x*sqrt(a-x^2)",
	"-1/3*sqrt((a-x^2)^3)",
	NULL,
//210
	"x^2*sqrt(a-x^2)",
	"-x/4*sqrt((a-x^2)^3)+1/8*a*(x*sqrt(a-x^2)+a*arcsin(x/sqrt(a)))",
	"a>0",
	NULL,

//211
	"x^3*sqrt(a-x^2)",
	"(-1/5*x^2-2/15*a)*sqrt((a-x^2)^3)",
	"a>0",
	NULL,

	"x^2/sqrt(a-x^2)",
	"-x/2*sqrt(a-x^2)+a/2*arcsin(x/sqrt(a))",
	"a>0",
	NULL,

	"1/x^2*1/sqrt(a-x^2)",
	"-sqrt(a-x^2)/a/x",
	"a>0",
	NULL,
//216
	"sqrt(a-x^2)/x^2",
	"-sqrt(a-x^2)/x-arcsin(x/sqrt(a))",
	"a>0",
	NULL,
//217
	"sqrt(a-x^2)/x^3",
	"-1/2*sqrt(a-x^2)/x^2+1/2*log((sqrt(a)+sqrt(a-x^2))/x)/sqrt(a)",
	"a>0",
	NULL,
//218
	"sqrt(a-x^2)/x^4",
	"-1/3*sqrt((a-x^2)^3)/a/x^3",
	"a>0",
	NULL,
// 290

	"sin(a*x)",
	"-cos(a*x)/a",
	NULL,

	"x^n*sin(a*x)",
	"x^n*antider(sin(a*x),x)-n*antider(antider(sin(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a*x)^2",
	"antider(1/2-1/2*cos(2*a*x),x)",
	NULL,

	"sin(a+b*x)",
	"-cos(a+b*x)/b",
	NULL,

	"x^n*sin(a*x+b)",
	"x^n*antider(sin(a*x+b),x)-n*antider(antider(sin(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a+b*x)^2",
	"antider(1/2-1/2*cos(2*(a+b*x)),x)",
	NULL,

	"cos(a*x)",
	"sin(a*x)/a",
	NULL,
	
	"x^n*cos(a*x)",
	"x^n*antider(cos(a*x),x)-n*antider(antider(cos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a*x)^2",
	"antider(1/2+1/2*cos(2*a*x),x)",
	NULL,

	"cos(a+b*x)",
	"sin(a+b*x)/b",
	NULL,
	
	"x^n*cos(a*x+b)",
	"x^n*antider(cos(a*x+b),x)-n*antider(antider(cos(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a+b*x)^2",
	"antider(1/2+1/2*cos(2*(a+b*x)),x)",
	NULL,

	"sin(a*x)/cos(a*x)",
	"-log(cos(a*x))/a",
	NULL,

	"x^n*sin(a*x)/cos(a*x)",
	"x^n*antider(sin(a*x)/cos(a*x),x)-n*antider(antider(sin(a*x)/cos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"tan(a*x)",
	"-log(cos(a*x))/a",
	NULL,

	"x^n*tan(a*x)",
	"x^n*antider(tan(a*x),x)-n*antider(antider(tan(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a+b*x)/cos(a+b*x)",
	"-log(cos(a+b*x))/b",
	NULL,

	"x^n*sin(a*x+b)/cos(a*x+b)",
	"x^n*antider(sin(a*x+b)/cos(a*x+b),x)-n*antider(antider(sin(a*x+b)/cos(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"tan(a+b*x)",
	"-log(cos(a+b*x))/b",
	NULL,

	"x^n*tan(a*x+b)",
	"x^n*antider(tan(a*x+b),x)-n*antider(antider(tan(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a*x)/sin(a*x)",
	"log(sin(a*x))/a",
	NULL,

	"x^n*cos(a*x)/sin(a*x)",
	"x^n*antider(cos(a*x)/sin(a*x),x)-n*antider(antider(cos(a*x)/sin(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/tan(a*x)",
	"log(sin(a*x))/a",
	NULL,

	"x^n/tan(a*x)",
	"x^n*antider(1/tan(a*x),x)-n*antider(antider(1/tan(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a*x+b)/sin(a*x+b)",
	"log(sin(a*x+b))/a",
	NULL,

	"x^n*cos(a*x+b)/sin(a*x+b)",
	"x^n*antider(cos(a*x+b)/sin(a*x+b),x)-n*antider(antider(cos(a*x+b)/sin(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/tan(a*x+b)",
	"log(sin(a*x+b))/a",
	NULL,

	"x^n/tan(a*x+b)",
	"x^n*antider(1/tan(a*x+b),x)-n*antider(antider(1/tan(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/cos(a*x)",
	"log((sin(a*x)+1)/(sin(a*x)-1))/(2*a)",
	NULL,

	"x^n/cos(a*x)",
	"x^n*antider(1/cos(a*x),x)-n*antider(antider(1/cos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/cos(a*x+b)",
	"log((sin(a*x+b)+1)/(sin(a*x+b)-1))/(2*a)",
	NULL,

	"x^n/cos(a*x+b)",
	"x^n*antider(1/cos(a*x+b),x)-n*antider(antider(1/cos(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/sin(a*x)",
	"log((cos(a*x)-1)/(cos(a*x)+1))/(2*a)",
	NULL,
	
	"x^n/sin(a*x)",
	"x^n*antider(1/sin(a*x),x)-n*antider(antider(1/sin(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/sin(a*x+b)",
	"log((cos(a*x+b)-1)/(cos(a*x+b)+1))/(2*a)",
	NULL,

	"x^n/sin(a*x+b)",
	"x^n*antider(1/sin(a*x+b),x)-n*antider(antider(1/sin(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a*x)^2",
	"x/2-sin(2*a*x)/(4*a)",
	NULL,

	"sin(a*x+b)^2",
	"(a*x+b)/(2*a)-sin(2*(a*x+b))/(4*a)",
	NULL,

	"sin(a*x)^3",
	"-cos(a*x)*(sin(a*x)^2+2)/(3*a)",
	NULL,

	"sin(a*x+b)^3",
	"-cos(a*x+b)*(sin(a*x+b)^2+2)/(3*a)",
	NULL,

	"sin(a*x)^4",
	"3/8*x-sin(2*a*x)/(4*a)+sin(4*a*x)/(32*a)",
	NULL,

	"sin(a*x+b)^4",
	"3/(8*a)*(a*x+b)-sin(2*(a*x+b))/(4*a)+sin(4*(a*x+b))/(32*a)",
	NULL,

	"cos(a*x)^2",
	"x/2+sin(2*a*x)/(4*a)",
	NULL,

	"cos(a*x+b)^2",
	"(a*x+b)/(2*a)+sin(2*(a*x+b))/(4*a)",
	NULL,

	"cos(a*x)^3",
	"sin(a*x)*(cos(a*x)^2+2)/(3*a)",
	NULL,

	"cos(a*x+b)^3",
	"sin(a*x+b)*(cos(a*x+b)^2+2)/(3*a)",
	NULL,

	"cos(a*x)^4",
	"3/8*x+sin(2*a*x)/(4*a)+sin(4*a*x)/(32*a)",
	NULL,

	"cos(a*x+b)^4",
	"3/(8*a)*(a*x+b)+sin(2*(a*x+b))/(4*a)+sin(4*(a*x+b))/(32*a)",
	NULL,

	"1/sin(a*x)^2",
	"-1/(a*tan(a*x))",
	NULL,

	"1/cos(a*x)^2",
	"tan(a*x)/a",
	NULL,

	"sin(a*x)*cos(a*x)",
	"sin(a*x)^2/(2*a)",
	NULL,

	"x^n*sin(a*x)*cos(a*x)",
	"x^n*antider(sin(a*x)*cos(a*x),x)-n*antider(antider(sin(a*x)*cos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a*x)*cos(b*x)",
	"(b*sin(a*x)*sin(b*x)+a*cos(a*x)*cos(b*x))/(b^2-a^2)",
	NULL,

	"x^n*sin(a*x)*cos(b*x)",
	"x^n*antider(sin(a*x)*cos(b*x),x)-n*antider(antider(sin(a*x)*cos(b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"cos(a*x)*cos(b*x)",
	"sin((a+b)*x)/(2*(a+b))+sin((a-b)*x)/(2*(a-b))",
	NULL,

	"x^n*cos(a*x)*cos(b*x)",
	"x^n*antider(cos(a*x)*cos(b*x),x)-n*antider(antider(cos(a*x)*cos(b*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sin(a*x)^2*cos(a*x)^2",
	"-sin(4*a*x)/(32*a)+x/8",
	NULL,

	"sin(a*x)/cos(a*x)^2",
	"1/(a*cos(a*x))",
	NULL,

	"sin(a*x)^2/cos(a*x)",
	"(log(sin(a*x)+1)-log(sin(a*x)-1))/(2*a)-sin(a*x)/a",
	NULL,

	"cos(a*x)/sin(a*x)^2",
	"-1/(a*sin(a*x))",
	NULL,

	"1/(sin(a*x)*cos(a*x))",
	"log(tan(a*x))/a",
	NULL,

	"x^n/(sin(a*x)*cos(a*x))",
	"x^n*antider(1/(sin(a*x)*cos(a*x)),x)-n*antider(antider(1/(sin(a*x)*cos(a*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(sin(a*x)*cos(a*x)^2)",
	"(log(cos(a*x)-1)-log(cos(a*x)+1)+2/cos(a*x))/(2*a)",
	NULL,

	"1/(sin(a*x)^2*cos(a*x))",
	"(log(sin(a*x)+1)-log(sin(a*x)-1))/(2*a)-1/(sin(a*x)*a)",
	NULL,

	"1/(sin(a*x)^2*cos(a*x)^2)",
	"-2/(a*tan(2*a*x))",
	NULL,

	"exp(a*x+b)*sin(c*x+n)",
	"exp(a*x+b)*(-c*cos(c*x+n)+a*sin(c*x+n))/(a^2+c^2)",
	NULL,

	"exp(a*x+b)*sin(c*x+n)^2",
	"antider(exp(a*x+b)*(1/2-cos(2*(c*x+n))/2),x)",
	NULL,

	"exp(a*x)*sin(c*x+n)",
	"exp(a*x)*(-c*cos(c*x+n)+a*sin(c*x+n))/(a^2+c^2)",
	NULL,

	"exp(a*x)*sin(c*x+n)^2",
	"antider(exp(a*x)*(1/2-cos(2*(c*x+n))/2),x)",
	NULL,

	"exp(a*x+b)*sin(c*x)",
	"exp(a*x+b)*(-c*cos(c*x)+a*sin(c*x))/(a^2+c^2)",
	NULL,

	"x^n*exp(a*x+b)*sin(c*x)",
	"x^n*antider(exp(a*x+b)*sin(c*x),x)-n*antider(antider(exp(a*x+b)*sin(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b)*sin(c*x)^2",
	"antider(exp(a*x+b)*(1/2-cos(2*(c*x))/2),x)",
	NULL,

	"exp(a*x)*sin(c*x)",
	"exp(a*x)*(-c*cos(c*x)+a*sin(c*x))/(a^2+c^2)",
	NULL,

	"x^n*exp(a*x)*sin(c*x)",
	"x^n*antider(exp(a*x)*sin(c*x),x)-n*antider(antider(exp(a*x)*sin(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b)*cos(c*x+n)",
	"exp(a*x+b)*(a*cos(c*x+n)+c*sin(c*x+n))/(a^2+c^2)",
	NULL,

	"exp(a*x+b)*cos(c*x+n)^2",
	"antider(exp(a*x+b)*(1/2+cos(2*(c*x+n))/2),x)",
	NULL,

	"exp(a*x)*cos(c*x+n)",
	"exp(a*x)*(a*cos(c*x+n)+c*sin(c*x+n))/(a^2+c^2)",
	NULL,

	"exp(a*x)*cos(c*x+n)^2",
	"antider(exp(a*x)*(1/2+cos(2*(c*x+n))/2),x)",
	NULL,

	"exp(a*x)*cos(c*x)",
	"exp(a*x)*(a*cos(c*x)+c*sin(c*x))/(a^2+c^2)",
	NULL,

	"x^n*exp(a*x)*cos(c*x)",
	"x^n*antider(exp(a*x)*cos(c*x),x)-n*antider(antider(exp(a*x)*cos(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b)*cos(c*x)",
	"exp(a*x+b)*(a*cos(c*x)+c*sin(c*x))/(a^2+c^2)",
	NULL,

	"exp(a*x+b)*cos(c*x)^2",
	"antider(exp(a*x+b)*(1/2+cos(2*(c*x))/2),x)",
	NULL,

	"exp(a*x+b)*sh(c*x+n)",
	"exp(a*x+b)*(-c*ch(c*x+n)+a*sh(c*x+n))/(a^2-c^2)",
	NULL,

	"exp(a*x+b)*sh(c*x+n)^2",
	"antider(exp(a*x+b)*(ch(2*(c*x+n))-1)/2,x)",
	NULL,

	"exp(a*x)*sh(c*x+n)",
	"exp(a*x)*(-c*ch(c*x+n)+a*sh(c*x+n))/(a^2-c^2)",
	NULL,

	"exp(a*x)*sh(c*x+n)^2",
	"antider(exp(a*x)*(ch(2*(c*x+n))-1)/2,x)",
	NULL,

	"exp(a*x+b)*sh(c*x)",
	"exp(a*x+b)*(-c*ch(c*x)+a*sh(c*x))/(a^2-c^2)",
	NULL,

	"x^n*exp(a*x+b)*sh(c*x)",
	"x^n*antider(exp(a*x+b)*sh(c*x),x)-n*antider(antider(exp(a*x+b)*sh(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"x^n*exp(a*x)*sh(c*x)",
	"x^n*antider(exp(a*x)*sh(c*x),x)-n*antider(antider(exp(a*x)*sh(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b)*sh(c*x)^2",
	"antider(exp(a*x+b)*(ch(2*(c*x))-1)/2,x)",
	NULL,

	"exp(a*x)*sh(c*x)",
	"exp(a*x)*(-c*ch(c*x)+a*sh(c*x))/(a^2-c^2)",
	NULL,

	"exp(a*x)*sh(c*x)^2",
	"antider(exp(a*x)*(ch(2*(c*x))-1)/2,x)",
	NULL,

	"exp(a*x+b)*ch(c*x+n)",
	"exp(a*x+b)*(a*ch(c*x+n)-c*sh(c*x+n))/(a^2-c^2)",
	NULL,

	"exp(a*x+b)*ch(c*x+n)^2",
	"antider(exp(a*x+b)*(ch(2*(c*x+n))+1)/2,x)",
	NULL,

	"exp(a*x)*ch(c*x+n)",
	"exp(a*x)*(a*ch(c*x+n)-c*sh(c*x+n))/(a^2-c^2)",
	NULL,

	"exp(a*x)*ch(c*x+n)^2",
	"antider(exp(a*x)*(ch(2*(c*x+n))+1)/2,x)",
	NULL,

	"exp(a*x+b)*ch(c*x)",
	"exp(a*x+b)*(a*ch(c*x)-c*sh(c*x))/(a^2-c^2)",
	NULL,

	"x^n*exp(a*x+b)*sh(c*x)",
	"x^n*antider(exp(a*x+b)*sh(c*x),x)-n*antider(antider(exp(a*x+b)*sh(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x+b)*ch(c*x)^2",
	"antider(exp(a*x+b)*(ch(2*(c*x))+1)/2,x)",
	NULL,

	"exp(a*x)*ch(c*x)",
	"exp(a*x)*(a*ch(c*x)-c*sh(c*x))/(a^2-c^2)",
	NULL,

	"x^n*exp(a*x)*ch(c*x)",
	"x^n*antider(exp(a*x)*ch(c*x),x)-n*antider(antider(exp(a*x)*ch(c*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"exp(a*x)*ch(c*x)^2",
	"antider(exp(a*x)*(ch(2*(c*x))+1)/2,x)",
	NULL,

 	"1/(a-a*sin(b*x))",
 	"-2/((a*b)*(tan(b*x/2)-1))",
 	NULL,

	"x^n/(a-a*sin(b*x))",
	"x^n*antider(1/(a-a*sin(b*x)),x)-n*antider(antider(1/(a-a*sin(b*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+a*sin(b*x))",
	"-2/((a*b)*(tan(b*x/2)+1))",
 	NULL,

	"x^n/(a+a*sin(b*x))",
	"x^n*antider(1/(a+a*sin(b*x)),x)-n*antider(antider(1/(a+a*sin(b*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+b*sin(c*x))",
	"2*arctan((b+a*tan(c*x/2))/sqrt(a^2-b^2))/(c*sqrt(a^2-b^2))",
	"b^2<a^2",
	NULL,

	"1/(a+b*sin(c*x))",
	"log((a*tan(c*x/2)-sqrt(b^2-a^2)+b)/(a*tan(c*x/2)+sqrt(b^2-a^2)+b))/(c*sqrt(b^2-a^2))",
	"b^2>a^2",
	NULL,

	"x^n/(a+b*sin(c*x))",
	"x^n*antider(1/(a+b*sin(c*x)),x)-n*antider(antider(1/(a+b*sin(c*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(b+b*cos(a*x))",
	"tan(a*x/2)/(a*b)",
	NULL,

	"x^n/(b+b*cos(a*x))",
	"x^n*antider(1/(b+b*cos(a*x)),x)-n*antider(antider(1/(b+b*cos(a*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(b-b*cos(a*x))",
	"-1/(tan(a*x/2)*a*b)",
	NULL,

	"x^n/(b-b*cos(a*x))",
	"x^n*antider(1/(b-b*cos(a*x)),x)-n*antider(antider(1/(b-b*cos(a*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+b*cos(c*x))",
	"2*arctan((b-a)*tan(c*x/2)/sqrt(a^2-b^2))/(c*sqrt(a^2-b^2))",
	"b^2<a^2",
	NULL,

	"1/(a+b*cos(c*x))",
	"-log(((b-a)*tan(c*x/2)-sqrt(b^2-a^2))/((b-a)*tan(c*x/2)+sqrt(b^2-a^2)))/(c*sqrt(b^2-a^2))",
	"b^2>a^2",
	NULL,

	"x^n/(a+b*cos(c*x))",
	"x^n*antider(1/(a+b*cos(c*x)),x)-n*antider(antider(1/(a+b*cos(c*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"1/(a+b*tan(c*x))",
	"(a*c*x+b*log(a*cos(c*x)+b*sin(c*x)))/(c*(a^2+b^2))",
	NULL,	

	"x^n/(a+b*tan(c*x))",
	"x^n*antider(1/(a+b*tan(c*x)),x)-n*antider(antider(1/(a+b*tan(c*x)),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"arcsin(a*x)",
	"x*arcsin(a*x)+sqrt(1-a^2*x^2)/a",
	NULL,

	"x^n*arcsin(a*x)",
	"x^n*antider(arcsin(a*x),x)-n*antider(antider(arcsin(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"arcsin(a*x+b)",
	"((b+a*x)*arcsin(a*x+b)+sqrt(1-b^2-2*a*b*x-a^2*x^2))/a",
	NULL,

	"x^n*arcsin(a*x+b)",
	"x^n*antider(arcsin(a*x+b),x)-n*antider(antider(arcsin(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"argsh(a*x)",
	"x*argsh(a*x)-sqrt(1+a^2*x^2)/a",
	NULL,

	"x^n*argsh(a*x)",
	"x^n*antider(argsh(a*x),x)-n*antider(antider(argsh(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"argsh(a*x+b)",
	"((b+a*x)*argsh(a*x+b)-sqrt(1+b^2+2*a*b*x+a^2*x^2))/a",
	NULL,

	"x^n*argsh(a*x+b)",
	"x^n*antider(argsh(a*x+b),x)-n*antider(antider(argsh(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"arccos(a*x)",
	"x*arccos(a*x)-sqrt(1-a^2*x^2)/a",
	NULL,

	"x^n*arccos(a*x)",
	"x^n*antider(arccos(a*x),x)-n*antider(antider(arccos(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"arccos(a*x+b)",
	"-b*arcsin(a*x+b)/a+x*arccos(a*x+b)-sqrt(1-b^2-2*a*b*x-a^2*x^2)/a",
	NULL,

	"x^n*arccos(a*x+b)",
	"x^n*antider(arccos(a*x+b),x)-n*antider(antider(arccos(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"argch(a*x)",
	"x*argch(a*x)-sqrt((a*x)^2-1)/a",
	NULL,

	"x^n*argch(a*x)",
	"x^n*antider(argch(a*x),x)-n*antider(antider(argch(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"argch(a*x+b)",
	"(a*x+b)*argch(a*x+b)/a-sqrt((a*x+b)^2-1)/a",
	NULL,

	"x^n*argch(a*x+b)",
	"x^n*antider(argch(a*x+b),x)-n*antider(antider(argch(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"arctan(a*x)",
	"x*arctan(a*x)-1/2*log(1+a^2*x^2)/a",
	NULL,

	"x*arctan(a*x)",
	"x^2*arctan(a*x)/2+arctan(a*x)/(2*a^2)-x/(2*a)",
	NULL,

	"x^n*arctan(a*x)",
	"x^n*antider(arctan(a*x),x)-n*antider(antider(arctan(a*x),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"arctan(a*x+b)",
	"x*arctan(a*x+b)+b*arctan(a*x+b)/a-log(1+b^2+2*a*b*x+a^2*x^2)/(2*a)",
	NULL,

	"x*arctan(a*x+b)",
	"b*log((a*x+b)^2+1)/(2*a^2)+x^2*arctan(a*x+b)/2-b^2*arctan(a*x+b)/(2*a^2)+arctan(a*x+b)/(2*a^2)-x/(2*a)",
	NULL,

	"x^n*arctan(a*x+b)",
	"x^n*antider(arctan(a*x+b),x)-n*antider(antider(arctan(a*x+b),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"argth(a*x)",
	"x*argth(a*x)+1/2*log(1-a^2*x^2)/a",
	NULL,

	"x*argth(a*x)",
	"1/a^2*(((a*x)^2-1)/2*argth(a*x)+a*x/2)",
	NULL,

	"x^n*argth(a*x)",
	"x^n*antider(argth(a*x),x)-n*antider(antider(argth(a*x),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"argth(a*x+b)",
	"x*argth(a*x+b)+1/2*((b+1)*log(a*x+b+1)-(b-1)*log(-a*x-b+1))/a",
	NULL,

	"x*argth(a*x+b)",
	"1/a^2*(((a*x+b)^2-1)/2*argth(a*x+b)+(a*x+b)/2)",
	NULL,

	"x^n*argth(a*x+b)",
	"x^n*antider(argth(a*x+b),x)-n*antider(antider(argth(a*x+b),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"log(a*x)",
	"x*log(a*x)-x",
	NULL,

	"x*log(a*x)",
	"x^2*log(a*x)/2-x^2/4",
	NULL,

	"x^2*log(a*x)",
	"x^3*log(a*x)/3-1/9*x^3",
	NULL,
	
	"x^n*log(a*x)",
	"x^n*antider(log(a*x),x)-n*antider(antider(log(a*x),x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"log(a*x+b)",
	"(a*x+b)*log(a*x+b)/a-x",
	NULL,

	"x*log(a*x+b)",
	"b*x/(2*a)-b^2*log(a*x+b)/(2*a^2)+x^2*log(a*x+b)/2-x^2/4",
	NULL,

	"x^2*log(a*x+b)",
	"-b^2*x/(3*a^2)+b*x^2/(6*a)+b^3*log(a*x+b)/(3*a^3)+x^3*log(a*x+b)/3-1/9*x^3",
	NULL,

	"x^n*log(a*x+b)",
	"x^n*antider(log(a*x+b),x)-n*antider(antider(log(a*x+b),x)*x^(n-1),x)",
	"(n>2)*(isinteger(n))",
	NULL,

	"x^a*log(x)^b",
	"(x^(a+1)*log(x)^b-b*antider(x^a*log(x)^(b-1),x))/(a+1)",
	"((isinteger(a))*not(a==-1)*(isinteger(b)))",
	NULL,
// 489
	"log(a*x)^2",
	"x*log(a*x)^2-2*x*log(a*x)+2*x",
	NULL,
	
	"log(a*x+b)^2",
	"((a*x+b)*log(a*x+b)^2-2*(a*x+b)*log(a*x+b)+2*(a*x+b))/a",
	NULL,

// 493 (with addition of a)
	"1/(x*log(a*x))",
	"log(log(a*x))",
	NULL,
// 500
	"log(a*x+b)/x^2",
	"a/b*log(x)-(a*x+b)*log(a*x+b)/b/x",
	NULL,

	"log(cos(a*x))*tan(a*x)",
	"-log(cos(a*x)^2)/(2*a)",
	NULL,

	"x^n*log(cos(a*x))*tan(a*x)",
	"x^n*antider(log(cos(a*x))*tan(a*x),x)-n*antider(antider(log(cos(a*x))*tan(a*x),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"log(cos(a*x+b))*tan(a*x+b)",
	"-log(cos(a*x+b)^2)/(2*a)",
	NULL,

	"x^n*log(cos(a*x+b))*tan(a*x+b)",
	"x^n*antider(log(cos(a*x+b))*tan(a*x+b),x)-n*antider(antider(log(cos(a*x+b))*tan(a*x+b),x)*x^(n-1),x)",
	"(n>0)*(isinteger(n))",
	NULL,

	"sh(a*x)",
	"ch(a*x)/a",
	NULL,
	
	"x*sh(a*x)",
	"x*ch(a*x)/a-sh(a*x)/a^2",
	NULL,

	"x^n*sh(a*x)",
	"x^n*antider(sh(a*x),x)-n*antider(antider(sh(a*x),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"sh(a*x+b)",
	"ch(a*x+b)/a",
	NULL,

	"x*sh(a*x+b)",
	"x*ch(a*x+b)/a-sh(a*x+b)/a^2",
	NULL,

	"x^n*sh(a*x+b)",
	"x^n*antider(sh(a*x+b),x)-n*antider(antider(sh(a*x+b),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"ch(a*x)",
	"sh(a*x)/a",
	NULL,
	
	"x*ch(a*x)",
	"x*sh(a*x)/a-ch(a*x)/a^2",
	NULL,

	"x^n*ch(a*x)",
	"x^n*antider(ch(a*x),x)-n*antider(antider(ch(a*x),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"ch(a*x+b)",
	"sh(a*x+b)/a",
	NULL,

	"x*ch(a*x+b)",
	"x*sh(a*x+b)/a-ch(a*x+b)/a^2",
	NULL,

	"x^n*ch(a*x+b)",
	"x^n*antider(ch(a*x+b),x)-n*antider(antider(ch(a*x+b),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"th(a*x)",
	"log(ch(a*x))/a",
	NULL,

	"x^n*th(a*x)",
	"x^n*antider(th(a*x),x)-n*antider(antider(th(a*x),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"th(a*x+b)",
	"log(ch(a*x+b))/a",
	NULL,

	"x^n*th(a*x+b)",
	"x^n*antider(th(a*x+b),x)-n*antider(antider(th(a*x+b),x)*x^(n-1),x)",
	"(n>1)*(isinteger(n))",
	NULL,

	"ch(a*x)^2",
	"sh(2*a*x)/(4*a)+x/2",
	NULL,

	"ch(a*x+b)^2",
	"sh(2*(a*x+b))/(4*a)+x/2",
	NULL,

	"sh(a*x)^2",
	"sh(2*a*x)/(4*a)-x/2",
	NULL,

	"sh(a*x+b)^2",
	"sh(2*(a*x+b))/(4*a)-(a*x+b)/(2*a)",
	NULL,

	"th(a*x)^2",
	"x-th(a*x)/a",
	NULL,

	"th(a*x+b)^2",
	"x-th(a*x+b)/a",
	NULL,

	"x^3*exp(a*x^2)",
	"exp(a*x^2)*(x^2/a-1/(a^2))/2",
	NULL,	
	
// ?
	"x^3*exp(a*x^2+b)",
	"exp(a*x^2)*exp(b)*(x^2/a-1/(a^2))/2",
	NULL,

	"1/(a*exp(c*x)+b*exp(-c*x))",
	"arctan(sqrt(a/b)*exp(c*x))/(sqrt(a*b)*c)",
	NULL,
	
	"1/(a+b*exp(c*x))",
	"x/a-log(a+b*exp(c*x))/(a*c)",
	NULL,

	"exp(c*x)/(a+b*exp(c*x))",
	"1/b*log(a+b*exp(c*x))",
	NULL,

	"1/sqrt(a+b*exp(c*x))",
	"2*arctan(sqrt(a+b*exp(c*x))/sqrt(-a))/(c*sqrt(-a))",
	"a<0",
	NULL,

	"1/sqrt(a+b*exp(c*x))",
	"log((sqrt(a+b*exp(c*x))-sqrt(a))/(sqrt(a+b*exp(c*x))+sqrt(a)))/(c*sqrt(a))",
	"a>0",
	NULL,

	"besselj(a*x,1)",
	"(1-besselj(a*x,0))/a",
	NULL,
	
	"besselj(a*x+b,1)",
	"(1-besselj(a*x+b,0))/a",
	NULL,

	"exp(a*x)/x^(1/2)",
	"erfi(sqrt(a*x))*sqrt(pi/a)",
	NULL,
	
	"exp(a*x+b)/x^(1/2)",
	"exp(b)*erfi(sqrt(a*x))*sqrt(pi/a)",
	NULL,

	"exp(a*x)*x^(1/2)",
	"exp(a*x)*sqrt(x)/a-erfi(sqrt(a*x))*sqrt(pi)/(2*a^(3/2))",
	NULL,	
	
	"exp(a*x+b)*x^(1/2)",
	"exp(a*x+b)*sqrt(x)/a-exp(b)*erfi(sqrt(a*x))*sqrt(pi)/(2*a^(3/2))",
	NULL,

	"erf(a*x)",
	"exp(-(a*x)^2)/(a*sqrt(pi))+x*erf(a*x)",
	NULL,

	"exp(a*x)*erf(b*x)",
	"(exp(a*x)*erf(b*x)-exp((a/(2*b))^2)*erf(b*x-a/(2*b)))/a",
	"not(a==0)",
	NULL,
	
	"erf(a*x+b)",
	"exp(-(a*x+b)^2)/(a*sqrt(pi))+(x+b/a)*erf(a*x+b)",
	NULL,
	
	"erfc(a*x)",
	"-exp(-(a*x)^2)/(a*sqrt(pi))+x*erfc(a*x)",
	NULL,

	"erfc(a*x+b)",
	"(a*x+b)/a*erfc(a*x+b)-exp(-(a*x+b)^2)/(a*sqrt(pi))",
	NULL,

// a<0	
	"exp(a*x^2)",
	"sqrt(pi/(-a))*erf(sqrt(-a)*x)/2",
	"conj(sqrt(-a))==sqrt(-a)",
	NULL,

// a>0
	"exp(a*x^2)",
	"sqrt(pi/a)*erfi(sqrt(a)*x)/2",
	"not(conj(sqrt(-a))==sqrt(-a))",
	NULL,

// a<0	
	"exp(a*x^2+b)",
	"exp(b)*sqrt(pi/(-a))*erf(sqrt(-a)*x)/2",
	"conj(sqrt(-a))==sqrt(-a)",
	NULL,

// a>0
	"exp(a*x^2+b)",
	"exp(b)*sqrt(pi/a)*erfi(sqrt(a)*x)/2",
	"not(conj(sqrt(-a))==sqrt(-a))",
	NULL,

// a<0
	"exp(a*x^2+b*x)",
	"sqrt(pi/(-a))*exp(-(b^2)/(4*a))*erf(sqrt(-a)*x-b/(2*sqrt(-a)))/2",
	"conj(sqrt(-a))==sqrt(-a)",
	NULL,
	
// a>0	
	"exp(a*x^2+b*x)",
	"sqrt(pi/a)*exp(-(b^2)/(4*a))*erfi(sqrt(a)*x+b/(2*sqrt(a)))/2",
	"not(conj(sqrt(-a))==sqrt(-a))",
	NULL,
	
// a<0
	"exp(a*x^2+b*x+c)",
	"exp(c)*sqrt(pi/(-a))*exp(-(b^2)/(4*a))*erf(sqrt(-a)*x-b/(2*sqrt(-a)))/2",
	"conj(sqrt(-a))==sqrt(-a)",
	NULL,
	
// a>0	
	"exp(a*x^2+b*x+c)",
	"exp(c)*sqrt(pi/a)*exp(-(b^2)/(4*a))*erfi(sqrt(a)*x+b/(2*sqrt(a)))/2",
	"not(conj(sqrt(-a))==sqrt(-a))",
	NULL,	

	"exp(a*x^2+b/x^2)",
	"sqrt(pi)*exp(-2*sqrt(-a)*sqrt(-b))/(4*sqrt(-a))*(erfc(sqrt(-b)/x-sqrt(-a)*x)-exp(4*sqrt(-a)*sqrt(-b))*erfc(sqrt(-a)*x+sqrt(-b)/x))",
	NULL,

	};
// build the table, subst. meta symbols for a, b and x.

static void
scan_integrals(void)
{
	int h, i, k;

	k = tos;

	i = 0;

	while (integrals[i]) {

		h = tos;

		while (integrals[i]) {
			scan(integrals[i++]);
			push_symbol(SYMBOL_A);
			push(meta_a);
			subst();
			push_symbol(SYMBOL_B);
			push(meta_b);
			subst();
			push_symbol(SYMBOL_C);
			push(meta_c);
			subst();
			push_symbol(SYMBOL_N);
			push(meta_n);
			subst();
			push_symbol(SYMBOL_X);
			push(meta_x);
			subst();
		}

		list(tos - h);

		i++;
	}

	list(tos - k);

	table_of_integrals = pop();  
}


